@@ -4,37 +4,17 @@
  * Based on routines from tcpdump patches by
  *   Ken Hornstein <kenh@cmf.nrl.navy.mil>
  *
- * $Id: packet-rx.c 18196 2006-05-21 04:49:01Z sahlberg $
- *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "config.h"
 
-#include <stdio.h>
-
-#include <string.h>
-#include <glib.h>
 #include <epan/packet.h>
 #include "packet-rx.h"
 #include <epan/addr_resolv.h>
@@ -47,10 +27,12 @@
  * XXX - is the "Epoch" really a UN*X time?  The high-order bit, according
  * to that spec, is a flag bit.
  */
+void proto_register_rx(void);
+void proto_reg_handoff_rx(void);
 
-#define UDP_PORT_RX_LOW		7000
-#define UDP_PORT_RX_HIGH	7009
-#define UDP_PORT_RX_AFS_BACKUPS	7021
+static dissector_handle_t rx_handle;
+
+#define UDP_PORT_RX_RANGE	"7000-7009,7021"
 
 static const value_string rx_types[] = {
 	{ RX_PACKET_TYPE_DATA,		"data" },
@@ -147,29 +129,22 @@ dissect_rx_response_encrypted(tvbuff_t *tvb, proto_tree *parent_tree, int offset
 	int i;
 	guint32 callnumber;
 
-	item = proto_tree_add_item(parent_tree, hf_rx_encrypted, tvb, offset, -1, FALSE);
+	item = proto_tree_add_item(parent_tree, hf_rx_encrypted, tvb, offset, -1, ENC_NA);
 	tree = proto_item_add_subtree(item, ett_rx_encrypted);
 
 	/* epoch : 4 bytes */
-	{
-		nstime_t ts;
-		ts.secs = tvb_get_ntohl(tvb, offset);
-		ts.nsecs = 0;
-
-		proto_tree_add_time(tree, hf_rx_epoch, tvb,
-			offset, 4, &ts);
-		offset += 4;
-	}
-
-	/* cid : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_epoch, tvb, offset, 4, ENC_TIME_SECS|ENC_BIG_ENDIAN);
 	offset += 4;
 
-	/*FIXME dont know how to handle this checksum, skipping it */
+	/* cid : 4 bytes */
+	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, ENC_BIG_ENDIAN);
+	offset += 4;
+
+	/*FIXME don't know how to handle this checksum, skipping it */
 	offset += 4;
 
 	/* sequrityindex : 1 byte */
-	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, FALSE);
+	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	for (i=0; i<RX_MAXCALLS; i++) {
@@ -181,11 +156,11 @@ dissect_rx_response_encrypted(tvbuff_t *tvb, proto_tree *parent_tree, int offset
 	}
 
 	/* inc nonce : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_inc_nonce, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_inc_nonce, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* level : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_level, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_level, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	proto_item_set_len(item, offset-old_offset);
@@ -201,8 +176,7 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	guint32 version, tl;
 	int old_offset=offset;
 
-	if (check_col(pinfo->cinfo, COL_INFO)) {
-		col_add_fstr(pinfo->cinfo, COL_INFO,
+	col_add_fstr(pinfo->cinfo, COL_INFO,
 			"RESPONSE  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -210,12 +184,11 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 			"Destination Port: %s  ",
 			(unsigned long)seq,
 			(unsigned long)callnumber,
-			get_udp_port(pinfo->srcport),
-			get_udp_port(pinfo->destport)
+			udp_port_to_display(pinfo->pool, pinfo->srcport),
+			udp_port_to_display(pinfo->pool, pinfo->destport)
 		);
-	}
 
-	item = proto_tree_add_item(parent_tree, hf_rx_response, tvb, offset, -1, FALSE);
+	item = proto_tree_add_item(parent_tree, hf_rx_response, tvb, offset, -1, ENC_NA);
 	tree = proto_item_add_subtree(item, ett_rx_response);
 
 	version = tvb_get_ntohl(tvb, offset);
@@ -231,7 +204,7 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 		offset = dissect_rx_response_encrypted(tvb, tree, offset);
 
 		/* kvno */
-		proto_tree_add_item(tree, hf_rx_kvno, tvb, offset, 4, FALSE);
+		proto_tree_add_item(tree, hf_rx_kvno, tvb, offset, 4, ENC_BIG_ENDIAN);
 		offset += 4;
 
 		/* ticket_len */
@@ -240,8 +213,7 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 			offset, 4, tl);
 		offset += 4;
 
-		tvb_ensure_bytes_exist(tvb, offset, tl);
-		proto_tree_add_item(tree, hf_rx_ticket, tvb, offset, tl, FALSE);
+		proto_tree_add_item(tree, hf_rx_ticket, tvb, offset, tl, ENC_NA);
 		offset += tl;
 	}
 
@@ -256,8 +228,7 @@ dissect_rx_abort(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 	proto_item *item;
 	int old_offset=offset;
 
-	if (check_col(pinfo->cinfo, COL_INFO)) {
-		col_add_fstr(pinfo->cinfo, COL_INFO,
+	col_add_fstr(pinfo->cinfo, COL_INFO,
 			"ABORT  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -265,16 +236,15 @@ dissect_rx_abort(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 			"Destination Port: %s  ",
 			(unsigned long)seq,
 			(unsigned long)callnumber,
-			get_udp_port(pinfo->srcport),
-			get_udp_port(pinfo->destport)
+			udp_port_to_display(pinfo->pool, pinfo->srcport),
+			udp_port_to_display(pinfo->pool, pinfo->destport)
 		);
-	}
 
-	item = proto_tree_add_item(parent_tree, hf_rx_abort, tvb, offset, -1, FALSE);
+	item = proto_tree_add_item(parent_tree, hf_rx_abort, tvb, offset, -1, ENC_NA);
 	tree = proto_item_add_subtree(item, ett_rx_abort);
 
-	/* kvno */
-	proto_tree_add_item(tree, hf_rx_abortcode, tvb, offset, 4, FALSE);
+	/* abort code */
+	proto_tree_add_item(tree, hf_rx_abortcode, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	proto_item_set_len(item, offset-old_offset);
@@ -290,8 +260,7 @@ dissect_rx_challenge(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	guint32 version;
 	int old_offset=offset;
 
-	if (check_col(pinfo->cinfo, COL_INFO)) {
-		col_add_fstr(pinfo->cinfo, COL_INFO,
+	col_add_fstr(pinfo->cinfo, COL_INFO,
 			"CHALLENGE  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -299,12 +268,11 @@ dissect_rx_challenge(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 			"Destination Port: %s  ",
 			(unsigned long)seq,
 			(unsigned long)callnumber,
-			get_udp_port(pinfo->srcport),
-			get_udp_port(pinfo->destport)
+			udp_port_to_display(pinfo->pool, pinfo->srcport),
+			udp_port_to_display(pinfo->pool, pinfo->destport)
 		);
-	}
 
-	item = proto_tree_add_item(parent_tree, hf_rx_challenge, tvb, offset, -1, FALSE);
+	item = proto_tree_add_item(parent_tree, hf_rx_challenge, tvb, offset, -1, ENC_NA);
 	tree = proto_item_add_subtree(item, ett_rx_challenge);
 
 	version = tvb_get_ntohl(tvb, offset);
@@ -313,10 +281,10 @@ dissect_rx_challenge(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	offset += 4;
 
 	if (version==2) {
-		proto_tree_add_item(tree, hf_rx_nonce, tvb, offset, 4, FALSE);
+		proto_tree_add_item(tree, hf_rx_nonce, tvb, offset, 4, ENC_BIG_ENDIAN);
 		offset += 4;
 
-		proto_tree_add_item(tree, hf_rx_min_level, tvb, offset, 4, FALSE);
+		proto_tree_add_item(tree, hf_rx_min_level, tvb, offset, 4, ENC_BIG_ENDIAN);
 		offset += 4;
 	}
 
@@ -329,49 +297,38 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 {
 	proto_tree *tree;
 	proto_item *item;
-	guint8 num;
+	guint8 num, reason;
 	int old_offset = offset;
 
-	if (check_col(pinfo->cinfo, COL_INFO)) {
-		col_add_fstr(pinfo->cinfo, COL_INFO,
-			"ACK  "
-			"Seq: %lu  "
-			"Call: %lu  "
-			"Source Port: %s  "
-			"Destination Port: %s  ",
-			(unsigned long)seq,
-			(unsigned long)callnumber,
-			get_udp_port(pinfo->srcport),
-			get_udp_port(pinfo->destport)
-		);
-	}
 
-	item = proto_tree_add_item(parent_tree, hf_rx_ack, tvb, offset, -1, FALSE);
+
+	item = proto_tree_add_item(parent_tree, hf_rx_ack, tvb, offset, -1, ENC_NA);
 	tree = proto_item_add_subtree(item, ett_rx_ack);
 
 
 	/* bufferspace: 2 bytes*/
-	proto_tree_add_item(tree, hf_rx_bufferspace, tvb, offset, 2, FALSE);
+	proto_tree_add_item(tree, hf_rx_bufferspace, tvb, offset, 2, ENC_BIG_ENDIAN);
 	offset += 2;
 
 	/* maxskew: 2 bytes*/
-	proto_tree_add_item(tree, hf_rx_maxskew, tvb, offset, 2, FALSE);
+	proto_tree_add_item(tree, hf_rx_maxskew, tvb, offset, 2, ENC_BIG_ENDIAN);
 	offset += 2;
 
 	/* first packet: 4 bytes*/
-	proto_tree_add_item(tree, hf_rx_first_packet, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_first_packet, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* prev packet: 4 bytes*/
-	proto_tree_add_item(tree, hf_rx_prev_packet, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_prev_packet, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* serial : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* reason : 1 byte */
-	proto_tree_add_item(tree, hf_rx_reason, tvb, offset, 1, FALSE);
+	reason = tvb_get_guint8(tvb, offset);
+	proto_tree_add_item(tree, hf_rx_reason, tvb, offset, 1, ENC_BIG_ENDIAN);
 	offset += 1;
 
 	/* nACKs */
@@ -381,43 +338,57 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 
 	while(num--){
 		proto_tree_add_item(tree, hf_rx_ack_type, tvb, offset, 1,
-			FALSE);
+			ENC_BIG_ENDIAN);
 		offset += 1;
 	}
 
-	/* Some implementations adds some extra fields.
+	/* Some implementations add some extra fields.
 	 * As far as I can see, these first add 3 padding bytes and then
-         * up to 4 32-bit values. (0,3,4 have been witnessed)
+	 * up to 4 32-bit values. (0,3,4 have been witnessed)
 	 *
-	 * RX as a protocol seems to be completely nondefined and seems to lack
+	 * RX as a protocol seems to be completely undefined and seems to lack
 	 * any sort of documentation other than "read the source of any of the
-	 * (compatible?) implementations.
-         */
-	if (tvb_length_remaining(tvb, offset)>3) {
-		offset += 3;	/* guess. some implementations adds 3 bytes */
+	 * (compatible?) implementations.  The OpenAFS source indicates that
+	 * 3 bytes of padding are written after the acks.
+	 */
+	if (tvb_reported_length_remaining(tvb, offset)>3) {
+		offset += 3;	/* guess. some implementations add 3 bytes */
 
 		if (tvb_reported_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_item(tree, hf_rx_ifmtu, tvb, offset, 4,
-				FALSE);
+			proto_tree_add_item(tree, hf_rx_maxmtu, tvb, offset, 4,
+				ENC_BIG_ENDIAN);
 			offset += 4;
 		}
 		if (tvb_reported_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_item(tree, hf_rx_maxmtu, tvb, offset, 4,
-				FALSE);
+			proto_tree_add_item(tree, hf_rx_ifmtu, tvb, offset, 4,
+				ENC_BIG_ENDIAN);
 			offset += 4;
 		}
 		if (tvb_reported_length_remaining(tvb, offset) >= 4){
 			proto_tree_add_item(tree, hf_rx_rwind, tvb, offset, 4,
-				FALSE);
+				ENC_BIG_ENDIAN);
 			offset += 4;
 		}
 		if (tvb_reported_length_remaining(tvb, offset) >= 4){
 			proto_tree_add_item(tree, hf_rx_maxpackets, tvb, offset, 4,
-				FALSE);
+				ENC_BIG_ENDIAN);
 			offset += 4;
 		}
 	}
 
+	col_add_fstr(pinfo->cinfo, COL_INFO,
+			"ACK %s  "
+			"Seq: %lu  "
+			"Call: %lu  "
+			"Source Port: %s  "
+			"Destination Port: %s  ",
+			val_to_str(reason, rx_reason, "%d"),
+			(unsigned long)seq,
+			(unsigned long)callnumber,
+			udp_port_to_display(pinfo->pool, pinfo->srcport),
+			udp_port_to_display(pinfo->pool, pinfo->destport)
+		);
+
 	proto_item_set_len(item, offset-old_offset);
 	return offset;
 }
@@ -426,65 +397,62 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 static int
 dissect_rx_flags(tvbuff_t *tvb, struct rxinfo *rxinfo, proto_tree *parent_tree, int offset)
 {
-	proto_tree *tree;
-	proto_item *item;
-	guint8 flags;
+	static int * const flags[] = {
+		&hf_rx_flags_free_packet,
+		&hf_rx_flags_more_packets,
+		&hf_rx_flags_last_packet,
+		&hf_rx_flags_request_ack,
+		&hf_rx_flags_clientinit,
+		NULL
+	};
 
-	flags = tvb_get_guint8(tvb, offset);
-	rxinfo->flags = flags;
+	rxinfo->flags = tvb_get_guint8(tvb, offset);
 
-	item = proto_tree_add_uint(parent_tree, hf_rx_flags, tvb,
-		offset, 1, flags);
-	tree = proto_item_add_subtree(item, ett_rx_flags);
-
-	proto_tree_add_boolean(tree, hf_rx_flags_free_packet, tvb,
-		offset, 1, flags);
-	proto_tree_add_boolean(tree, hf_rx_flags_more_packets, tvb,
-		offset, 1, flags);
-	proto_tree_add_boolean(tree, hf_rx_flags_last_packet, tvb,
-		offset, 1, flags);
-	proto_tree_add_boolean(tree, hf_rx_flags_request_ack, tvb,
-		offset, 1, flags);
-	proto_tree_add_boolean(tree, hf_rx_flags_clientinit, tvb,
-		offset, 1, flags);
+	proto_tree_add_bitmask(parent_tree, tvb, offset, hf_rx_flags, ett_rx_flags, flags, ENC_NA);
 
 	offset += 1;
 	return offset;
 }
 
-static void
-dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
+static int
+dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, void *data _U_)
 {
 	proto_tree *tree;
 	proto_item *item;
+	const char *version_type;
 	int offset = 0;
 	struct rxinfo rxinfo;
 	guint8 type;
+	nstime_t ts;
 	guint32 seq, callnumber;
 	guint16 serviceid;
 
-	if (check_col(pinfo->cinfo, COL_PROTOCOL))
-		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RX");
-	if (check_col(pinfo->cinfo, COL_INFO))
-		col_clear(pinfo->cinfo, COL_INFO);
+	/* Ensure we have enough data */
+	if (tvb_captured_length(tvb) < 28)
+		return 0;
+
+	/* Make sure it's a known type */
+	type = tvb_get_guint8(tvb, 20);
+	if (!try_val_to_str(type, rx_types))
+		return 0;
+
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "RX");
+	col_clear(pinfo->cinfo, COL_INFO);
 
 	item = proto_tree_add_protocol_format(parent_tree, proto_rx, tvb,
 		offset,	28, "RX Protocol");
 	tree = proto_item_add_subtree(item, ett_rx);
 
 	/* epoch : 4 bytes */
-	{
-		nstime_t ts;
-		ts.secs = tvb_get_ntohl(tvb, offset);
-		ts.nsecs = 0;
-
-		proto_tree_add_time(tree, hf_rx_epoch, tvb,
-			offset, 4, &ts);
-		offset += 4;
-	}
+	rxinfo.epoch = tvb_get_ntohl(tvb, offset);
+	ts.secs = rxinfo.epoch;
+	ts.nsecs = 0;
+	proto_tree_add_time(tree, hf_rx_epoch, tvb, offset, 4, &ts);
+	offset += 4;
 
 	/* cid : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, FALSE);
+	rxinfo.cid = tvb_get_ntohl(tvb, offset);
+	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* callnumber : 4 bytes */
@@ -502,7 +470,7 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	rxinfo.seq = seq;
 
 	/* serial : 4 bytes */
-	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* type : 1 byte */
@@ -516,11 +484,11 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	offset = dissect_rx_flags(tvb, &rxinfo, tree, offset);
 
 	/* userstatus : 1 byte */
-	proto_tree_add_item(tree, hf_rx_userstatus, tvb, offset, 1, FALSE);
+	proto_tree_add_item(tree, hf_rx_userstatus, tvb, offset, 1, ENC_BIG_ENDIAN);
 	offset += 1;
 
 	/* sequrityindex : 1 byte */
-	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, FALSE);
+	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, ENC_BIG_ENDIAN);
 	offset += 1;
 
 	/*
@@ -530,7 +498,7 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	 */
 
 	/* spare */
-	proto_tree_add_item(tree, hf_rx_spare, tvb, offset, 2, FALSE);
+	proto_tree_add_item(tree, hf_rx_spare, tvb, offset, 2, ENC_BIG_ENDIAN);
 	offset += 2;
 
 	/* service id : 2 bytes */
@@ -543,15 +511,14 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	switch (type) {
 	case RX_PACKET_TYPE_ACK:
 		/*dissect_rx_acks(tvb, pinfo, parent_tree, offset,
-			cant create it in a parallell tree, then ett seasrch
-			wont work */
+			can't create it in a parallel tree, then ett search
+			won't work */
 		dissect_rx_acks(tvb, pinfo, tree, offset,
 			seq, callnumber);
 		break;
 	case RX_PACKET_TYPE_ACKALL:
 		/* does not contain any payload */
-		if (check_col(pinfo->cinfo, COL_INFO)) {
-			col_add_fstr(pinfo->cinfo, COL_INFO,
+		col_add_fstr(pinfo->cinfo, COL_INFO,
 				"ACKALL  "
 				"Seq: %lu  "
 				"Call: %lu  "
@@ -559,10 +526,29 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 				"Destination Port: %s  ",
 				(unsigned long)seq,
 				(unsigned long)callnumber,
-				get_udp_port(pinfo->srcport),
-				get_udp_port(pinfo->destport)
+				udp_port_to_display(pinfo->pool, pinfo->srcport),
+				udp_port_to_display(pinfo->pool, pinfo->destport)
+			);
+		break;
+	case RX_PACKET_TYPE_VERSION:
+		/* does not contain any payload */
+		if (rxinfo.cid == 0)
+		    version_type = "NAT ping";
+		else
+		    version_type = "request";
+
+		col_add_fstr(pinfo->cinfo, COL_INFO,
+				"VERSION %s  "
+				"Seq: %lu  "
+				"Call: %lu  "
+				"Source Port: %s  "
+				"Destination Port: %s  ",
+				version_type,
+				(unsigned long)seq,
+				(unsigned long)callnumber,
+				udp_port_to_display(pinfo->pool, pinfo->srcport),
+				udp_port_to_display(pinfo->pool, pinfo->destport)
 			);
-		}
 		break;
 	case RX_PACKET_TYPE_CHALLENGE:
 		dissect_rx_challenge(tvb, pinfo, tree, offset, seq, callnumber);
@@ -572,9 +558,9 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		break;
 	case RX_PACKET_TYPE_DATA: {
 		tvbuff_t *next_tvb;
-		pinfo->private_data = &rxinfo;
-		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-		call_dissector(afs_handle, next_tvb, pinfo, parent_tree);
+
+		next_tvb = tvb_new_subset_remaining(tvb, offset);
+		call_dissector_with_data(afs_handle, next_tvb, pinfo, parent_tree, &rxinfo);
 		};
 		break;
 	case RX_PACKET_TYPE_ABORT:
@@ -582,6 +568,7 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		break;
 	}
 
+	return(tvb_captured_length(tvb));
 }
 
 void
@@ -589,70 +576,70 @@ proto_register_rx(void)
 {
 	static hf_register_info hf[] = {
 		{ &hf_rx_epoch, {
-			"Epoch", "rx.epoch", FT_ABSOLUTE_TIME, BASE_DEC,
-			NULL, 0, "Epoch", HFILL }},
+			"Epoch", "rx.epoch", FT_ABSOLUTE_TIME, ABSOLUTE_TIME_LOCAL,
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_cid, {
 			"CID", "rx.cid", FT_UINT32, BASE_DEC,
-			NULL, 0, "CID", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_callnumber, {
 			"Call Number", "rx.callnumber", FT_UINT32, BASE_DEC,
-			NULL, 0, "Call Number", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_seq, {
 			"Sequence Number", "rx.seq", FT_UINT32, BASE_DEC,
-			NULL, 0, "Sequence Number", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_serial, {
 			"Serial", "rx.serial", FT_UINT32, BASE_DEC,
-			NULL, 0, "Serial", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_type, {
 			"Type", "rx.type", FT_UINT8, BASE_DEC,
-			VALS(rx_types), 0, "Type", HFILL }},
+			VALS(rx_types), 0, NULL, HFILL }},
 
 		{ &hf_rx_flags, {
 			"Flags", "rx.flags", FT_UINT8, BASE_HEX,
-			NULL, 0, "Flags", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_flags_clientinit, {
 			"Client Initiated", "rx.flags.client_init", FT_BOOLEAN, 8,
-			NULL, RX_CLIENT_INITIATED, "Client Initiated", HFILL }},
+			NULL, RX_CLIENT_INITIATED, NULL, HFILL }},
 
 		{ &hf_rx_flags_request_ack, {
 			"Request Ack", "rx.flags.request_ack", FT_BOOLEAN, 8,
-			NULL, RX_REQUEST_ACK, "Request Ack", HFILL }},
+			NULL, RX_REQUEST_ACK, NULL, HFILL }},
 
 		{ &hf_rx_flags_last_packet, {
 			"Last Packet", "rx.flags.last_packet", FT_BOOLEAN, 8,
-			NULL, RX_LAST_PACKET, "Last Packet", HFILL }},
+			NULL, RX_LAST_PACKET, NULL, HFILL }},
 
 		{ &hf_rx_flags_more_packets, {
 			"More Packets", "rx.flags.more_packets", FT_BOOLEAN, 8,
-			NULL, RX_MORE_PACKETS, "More Packets", HFILL }},
+			NULL, RX_MORE_PACKETS, NULL, HFILL }},
 
 		{ &hf_rx_flags_free_packet, {
 			"Free Packet", "rx.flags.free_packet", FT_BOOLEAN, 8,
-			NULL, RX_FREE_PACKET, "Free Packet", HFILL }},
+			NULL, RX_FREE_PACKET, NULL, HFILL }},
 
 		/* XXX - what about RX_SLOW_START_OR_JUMBO? */
 
 		{ &hf_rx_userstatus, {
 			"User Status", "rx.userstatus", FT_UINT32, BASE_DEC,
-			NULL, 0, "User Status", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_securityindex, {
 			"Security Index", "rx.securityindex", FT_UINT32, BASE_DEC,
-			NULL, 0, "Security Index", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_spare, {
 			"Spare/Checksum", "rx.spare", FT_UINT16, BASE_DEC,
-			NULL, 0, "Spare/Checksum", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_serviceid, {
 			"Service ID", "rx.serviceid", FT_UINT16, BASE_DEC,
-			NULL, 0, "Service ID", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_bufferspace, {
 			"Bufferspace", "rx.bufferspace", FT_UINT16, BASE_DEC,
@@ -660,11 +647,11 @@ proto_register_rx(void)
 
 		{ &hf_rx_maxskew, {
 			"Max Skew", "rx.maxskew", FT_UINT16, BASE_DEC,
-			NULL, 0, "Max Skew", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_first_packet, {
 			"First Packet", "rx.first", FT_UINT32, BASE_DEC,
-			NULL, 0, "First Packet", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_prev_packet, {
 			"Prev Packet", "rx.prev", FT_UINT32, BASE_DEC,
@@ -684,11 +671,11 @@ proto_register_rx(void)
 
 		{ &hf_rx_ack, {
 			"ACK Packet", "rx.ack", FT_NONE, BASE_NONE,
-			NULL, 0, "ACK Packet", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_challenge, {
 			"CHALLENGE Packet", "rx.challenge", FT_NONE, BASE_NONE,
-			NULL, 0, "CHALLENGE Packet", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_version, {
 			"Version", "rx.version", FT_UINT32, BASE_DEC,
@@ -696,7 +683,7 @@ proto_register_rx(void)
 
 		{ &hf_rx_nonce, {
 			"Nonce", "rx.nonce", FT_UINT32, BASE_HEX,
-			NULL, 0, "Nonce", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_inc_nonce, {
 			"Inc Nonce", "rx.inc_nonce", FT_UINT32, BASE_HEX,
@@ -704,19 +691,19 @@ proto_register_rx(void)
 
 		{ &hf_rx_min_level, {
 			"Min Level", "rx.min_level", FT_UINT32, BASE_DEC,
-			NULL, 0, "Min Level", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_level, {
 			"Level", "rx.level", FT_UINT32, BASE_DEC,
-			NULL, 0, "Level", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_response, {
 			"RESPONSE Packet", "rx.response", FT_NONE, BASE_NONE,
-			NULL, 0, "RESPONSE Packet", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_abort, {
 			"ABORT Packet", "rx.abort", FT_NONE, BASE_NONE,
-			NULL, 0, "ABORT Packet", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_encrypted, {
 			"Encrypted", "rx.encrypted", FT_NONE, BASE_NONE,
@@ -724,35 +711,35 @@ proto_register_rx(void)
 
 		{ &hf_rx_kvno, {
 			"kvno", "rx.kvno", FT_UINT32, BASE_DEC,
-			NULL, 0, "kvno", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_ticket_len, {
 			"Ticket len", "rx.ticket_len", FT_UINT32, BASE_DEC,
 			NULL, 0, "Ticket Length", HFILL }},
 
 		{ &hf_rx_ticket, {
-			"ticket", "rx.ticket", FT_BYTES, BASE_HEX,
-			NULL, 0, "Ticket", HFILL }},
+			"ticket", "rx.ticket", FT_BYTES, BASE_NONE,
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_ifmtu, {
 			"Interface MTU", "rx.if_mtu", FT_UINT32, BASE_DEC,
-			NULL, 0, "Interface MTU", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_maxmtu, {
 			"Max MTU", "rx.max_mtu", FT_UINT32, BASE_DEC,
-			NULL, 0, "Max MTU", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_rwind, {
 			"rwind", "rx.rwind", FT_UINT32, BASE_DEC,
-			NULL, 0, "rwind", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_maxpackets, {
 			"Max Packets", "rx.max_packets", FT_UINT32, BASE_DEC,
-			NULL, 0, "Max Packets", HFILL }},
+			NULL, 0, NULL, HFILL }},
 
 		{ &hf_rx_abortcode, {
-			"Abort Code", "rx.abort_code", FT_UINT32, BASE_DEC,
-			NULL, 0, "Abort Code", HFILL }},
+			"Abort Code", "rx.abort_code", FT_INT32, BASE_DEC,
+			NULL, 0, NULL, HFILL }},
 
 	};
 	static gint *ett[] = {
@@ -768,24 +755,30 @@ proto_register_rx(void)
 	proto_rx = proto_register_protocol("RX Protocol", "RX", "rx");
 	proto_register_field_array(proto_rx, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+
+	rx_handle = register_dissector("rx", dissect_rx, proto_rx);
 }
 
 void
 proto_reg_handoff_rx(void)
 {
-	dissector_handle_t rx_handle;
-
-	int port;
-
 	/*
 	 * Get handle for the AFS dissector.
 	 */
-	afs_handle = find_dissector("afs");
+	afs_handle = find_dissector_add_dependency("afs", proto_rx);
 
-	/* Ports in the range UDP_PORT_RX_LOW to UDP_PORT_RX_HIGH
-	   are all used for various AFS services. */
-	rx_handle = create_dissector_handle(dissect_rx, proto_rx);
-	for (port = UDP_PORT_RX_LOW; port <= UDP_PORT_RX_HIGH; port++)
-		dissector_add("udp.port", port, rx_handle);
-	dissector_add("udp.port", UDP_PORT_RX_AFS_BACKUPS, rx_handle);
+	dissector_add_uint_range_with_preference("udp.port", UDP_PORT_RX_RANGE, rx_handle);
 }
+
+/*
+ * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
+ *
+ * Local variables:
+ * c-basic-offset: 8
+ * tab-width: 8
+ * indent-tabs-mode: t
+ * End:
+ *
+ * vi: set shiftwidth=8 tabstop=8 noexpandtab:
+ * :indentSize=8:tabSize=8:noTabs=yes:
+ */
