@@ -2,39 +2,23 @@
  * Routines for SMB net logon packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-smb-logon.c 36054 2011-02-24 20:07:23Z gerald $
- *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-pop.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
-#include <glib.h>
+#include "config.h"
 
 #include <epan/packet.h>
 #include "packet-windows-common.h"
 #include "packet-smb-common.h"
 
+void proto_register_smb_logon(void);
+
 static int proto_smb_logon = -1;
 static int hf_command = -1;
 static int hf_computer_name = -1;
@@ -55,9 +39,11 @@ static int hf_lm_token = -1;
 static int hf_major_version = -1;
 static int hf_minor_version = -1;
 static int hf_os_version = -1;
+static int hf_signature = -1;
 static int hf_date_time = -1;
 static int hf_update_type = -1;
 static int hf_request_count = -1;
+static int hf_account_control = -1;
 static int hf_flags_autolock = -1;
 static int hf_flags_expire = -1;
 static int hf_flags_server_trust = -1;
@@ -86,22 +72,23 @@ static int hf_server_ip = -1;
 
 static int hf_server_site_name = -1;
 static int hf_client_site_name = -1;
+static int hf_data = -1;
 
 static int ett_smb_logon = -1;
 static int ett_smb_account_flags = -1;
 static int ett_smb_db_info = -1;
 
-#define	ACC_FLAG_AUTO_LOCKED			0x0400
-#define ACC_FLAG_EXPIRE				0x0200
-#define ACC_FLAG_SERVER_TRUST			0x0100
-#define ACC_FLAG_WORKSTATION_TRUST		0x0080
-#define ACC_FLAG_INTERDOMAIN_TRUST		0x0040
-#define ACC_FLAG_MNS_USER			0x0020
-#define ACC_FLAG_NORMAL_USER			0x0010
-#define ACC_FLAG_TEMP_DUP_USER			0x0008
-#define ACC_FLAG_PASSWORD_REQUIRED		0x0004
-#define ACC_FLAG_HOMEDIR_REQUIRED		0x0002
-#define ACC_FLAG_ENABLED			0x0001
+#define ACC_FLAG_AUTO_LOCKED        0x00000400
+#define ACC_FLAG_EXPIRE             0x00000200
+#define ACC_FLAG_SERVER_TRUST       0x00000100
+#define ACC_FLAG_WORKSTATION_TRUST  0x00000080
+#define ACC_FLAG_INTERDOMAIN_TRUST  0x00000040
+#define ACC_FLAG_MNS_USER           0x00000020
+#define ACC_FLAG_NORMAL_USER        0x00000010
+#define ACC_FLAG_TEMP_DUP_USER      0x00000008
+#define ACC_FLAG_PASSWORD_REQUIRED  0x00000004
+#define ACC_FLAG_HOMEDIR_REQUIRED   0x00000002
+#define ACC_FLAG_ENABLED            0x00000001
 
 static const true_false_string tfs_flags_autolock = {
 	"User account auto-locked",
@@ -154,34 +141,24 @@ static int
 dissect_account_control(tvbuff_t *tvb, proto_tree *tree, int offset)
 {
 	/* display the Allowable Account control bits */
+	static int * const flags[] = {
+		&hf_flags_autolock,
+		&hf_flags_expire,
+		&hf_flags_server_trust,
+		&hf_flags_workstation_trust,
+		&hf_flags_interdomain_trust,
+		&hf_flags_mns_user,
+		&hf_flags_normal_user,
+		&hf_flags_temp_dup_user,
+		&hf_flags_password_required,
+		&hf_flags_homedir_required,
+		&hf_flags_enabled,
+		NULL
+	};
 
-	proto_item *ti = NULL;
-	proto_tree *flags_tree = NULL;
-	guint32 flags;
-
-	flags = tvb_get_letohl(tvb, offset);
-
-	if (tree) {
-		ti = proto_tree_add_text(tree, tvb, offset, 4,
-			"Account control  = 0x%04x", flags);
-
-		flags_tree = proto_item_add_subtree(ti, ett_smb_account_flags);
-	}
-
-	proto_tree_add_boolean(flags_tree, hf_flags_autolock, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_expire, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_server_trust, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_workstation_trust, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_interdomain_trust, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_mns_user, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_normal_user, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_temp_dup_user, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_password_required, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_homedir_required, tvb, offset, 4, flags);
-	proto_tree_add_boolean(flags_tree, hf_flags_enabled, tvb, offset, 4, flags);
+	proto_tree_add_bitmask(tree, tvb, offset, hf_account_control, ett_smb_account_flags, flags, ENC_LITTLE_ENDIAN);
 
 	offset += 4;
-
 	return offset;
 }
 
@@ -220,9 +197,9 @@ display_LMNT_token(tvbuff_t *tvb, int offset, proto_tree *tree)
 	Token = tvb_get_letohs(tvb, offset);
 
 	if (Token == 0xffff) {
-		proto_tree_add_uint_format(tree, hf_lmnt_token, tvb, offset, 2,
+		proto_tree_add_uint_format_value(tree, hf_lmnt_token, tvb, offset, 2,
 			Token,
-			"LMNT Token: 0x%04x (Windows NT Networking)", Token);
+			"0x%04x (Windows NT Networking)", Token);
 	} else {
 		/*
 		 * XXX - what is it if it's not 0xffff?
@@ -238,27 +215,27 @@ display_LMNT_token(tvbuff_t *tvb, int offset, proto_tree *tree)
 }
 
 static int
-dissect_smb_logon_request(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_logon_request(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x00 (LM1.0/LM2.0 LOGON Request) ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_computer_name, NULL);
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_mailslot_name, NULL);
 
 	/*$$$$$ here add the Mailslot to the response list (if needed) */
 
 	/* Request count */
-	proto_tree_add_item(tree, hf_request_count, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_request_count, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* NT version */
-  	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 2, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 2, ENC_LITTLE_ENDIAN);
 	offset += 2;
 
 	/* LM token */
@@ -270,22 +247,22 @@ dissect_smb_logon_request(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 
 
 static int
-dissect_smb_logon_LM10_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_logon_LM10_resp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x01 LanMan 1.0 Logon response ***/
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* script name */
-	offset = display_ms_string(tvb, tree, offset, hf_script_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_script_name, NULL);
 
 	return offset;
 }
 
 
 static int
-dissect_smb_logon_2(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_logon_2(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x02  LM1.0 Query - Centralized Initialization ***/
 	/*** 0x03  LM1.0 Query - Distributed Initialization ***/
@@ -293,13 +270,13 @@ dissect_smb_logon_2(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 	/*** 0x04  LM1.0 Query - Distributed Query Response ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_computer_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_mailslot_name, NULL);
 
 	/* NT version */
-  	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 2, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 2, ENC_LITTLE_ENDIAN);
 	offset += 2;
 
 	/* LM token */
@@ -311,12 +288,12 @@ dissect_smb_logon_2(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 
 
 static int
-dissect_smb_logon_LM20_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_logon_LM20_resp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x06 (LM2.0 LOGON Response)	***/
 
 	/* server name */
-	offset = display_ms_string(tvb, tree, offset, hf_server_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_server_name, NULL);
 
 	/* LM token */
 	offset = display_LM_token(tvb, offset, tree);
@@ -327,20 +304,19 @@ dissect_smb_logon_LM20_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *t
 
 
 static int
-dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	char *name;
 
 	/*** 0x07 Query for Primary PDC  ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name, &name);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_computer_name, &name);
 
-	if (check_col(pinfo->cinfo, COL_INFO))
-		col_append_fstr(pinfo->cinfo, COL_INFO, " from %s", name);
+	col_append_fstr(pinfo->cinfo, COL_INFO, " from %s", name);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_mailslot_name, NULL);
 
 	if (tvb_reported_length_remaining(tvb, offset) > 2) {
 		/*
@@ -355,10 +331,10 @@ dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, i
 		if (offset % 2) offset++;      /* word align ... */
 
 		/* Unicode computer name */
-		offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name, NULL);
+		offset = display_unicode_string(tvb, pinfo, tree, offset, hf_unicode_computer_name, NULL);
 
 		/* NT version */
-	  	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	  	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 		offset += 4;
 
 		/* LMNT token */
@@ -374,12 +350,12 @@ dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, i
 
 
 static int
-dissect_smb_pdc_startup(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_pdc_startup(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x08  Announce startup of PDC ***/
 
 	/* pdc name */
-	offset = display_ms_string(tvb, tree, offset, hf_pdc_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_pdc_name, NULL);
 
 	/* A short Announce will not have the rest */
 
@@ -389,9 +365,9 @@ dissect_smb_pdc_startup(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	  if (offset % 2) offset++;      /* word align ... */
 
 	  /* pdc name */
-	  offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name, &name);
+	  offset = display_unicode_string(tvb, pinfo, tree, offset, hf_unicode_pdc_name, &name);
 
-	  if (name && check_col(pinfo->cinfo, COL_INFO)) {
+	  if (name) {
 		  col_append_fstr(pinfo->cinfo, COL_INFO, ": host %s", name);
 		  name = NULL;
 	  }
@@ -399,15 +375,15 @@ dissect_smb_pdc_startup(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	  if (offset % 2) offset++;
 
 	  /* domain name */
-	  offset = display_unicode_string(tvb, tree, offset, hf_domain_name, &name);
+	  offset = display_unicode_string(tvb, pinfo, tree, offset, hf_domain_name, &name);
 
-	  if (name && check_col(pinfo->cinfo, COL_INFO)) {
+	  if (name) {
 		  col_append_fstr(pinfo->cinfo, COL_INFO, ", domain %s", name);
 		  name = NULL;
 	  }
 
 	  /* NT version */
-  	  proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	  proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	  offset += 4;
 
 	  /* LMNT token */
@@ -430,7 +406,7 @@ dissect_smb_pdc_failure(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	/*** 0x10 (LM 2.0 Unknown user response) ***/
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LM token */
@@ -441,38 +417,37 @@ dissect_smb_pdc_failure(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 
 static int
-dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x0A ( Announce change to UAS or SAM ) ***/
 	guint32 info_count;
-	proto_item *ti = NULL;
-	proto_tree *info_tree = NULL;
+	proto_tree *info_tree;
 	guint32 db_index;
 	guint32 domain_sid_size;
 
 	/* low serial number */
-	proto_tree_add_item(tree, hf_low_serial, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_low_serial, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* date/time */
 	/* XXX - what format is this?  Neither SMB_Date/SMB_Time nor
 	   "time_t but in the local time zone" appear to be correct. */
-	proto_tree_add_item(tree, hf_date_time, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_date_time, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* pulse */
-	proto_tree_add_item(tree, hf_pulse, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_pulse, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* random */
-	proto_tree_add_item(tree, hf_random, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_random, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* pdc name */
-	offset = display_ms_string(tvb, tree, offset, hf_pdc_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_pdc_name, NULL);
 
 	/* domain name */
-	offset = display_ms_string(tvb, tree, offset, hf_domain_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_domain_name, NULL);
 
 	if (offset % 2) offset++;      /* word align ... */
 
@@ -481,10 +456,10 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 		 * XXX - older protocol versions don't have this stuff?
 		 */
 		/* pdc name */
-		offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name, NULL);
+		offset = display_unicode_string(tvb, pinfo, tree, offset, hf_unicode_pdc_name, NULL);
 
 		/* domain name */
-		offset = display_unicode_string(tvb, tree, offset, hf_domain_name, NULL);
+		offset = display_unicode_string(tvb, pinfo, tree, offset, hf_domain_name, NULL);
 
 		/* DB count */
 		info_count = tvb_get_letohl(tvb, offset);
@@ -493,18 +468,15 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 		while (info_count != 0) {
 			db_index = tvb_get_letohl(tvb, offset);
-			if (tree) {
-				ti = proto_tree_add_text(tree, tvb, offset, 20,
-				    "DBChange Info Structure: index %u", db_index);
-				info_tree = proto_item_add_subtree(ti, ett_smb_db_info);
-			}
+			info_tree = proto_tree_add_subtree_format(tree, tvb, offset, 20,
+				    ett_smb_db_info, NULL, "DBChange Info Structure: index %u", db_index);
 
 			proto_tree_add_uint(info_tree, hf_db_index, tvb, offset, 4,
 			    db_index);
 			offset += 4;
 
 			proto_tree_add_item(info_tree, hf_large_serial, tvb, offset, 8,
-			    TRUE);
+			    ENC_LITTLE_ENDIAN);
 			offset += 8;
 
 			offset = dissect_nt_64bit_time(tvb, info_tree, offset,
@@ -529,7 +501,7 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 		}
 
 		/* NT version */
-		proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+		proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 		offset += 4;
 
 		/* LMNT token */
@@ -544,24 +516,24 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 
 static int
-dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/* Netlogon command 0x12 - decode the SAM logon request from client */
 
 	guint32 domain_sid_size;
 
 	/* Request count */
-	proto_tree_add_item(tree, hf_request_count, tvb, offset, 2, TRUE);
+	proto_tree_add_item(tree, hf_request_count, tvb, offset, 2, ENC_LITTLE_ENDIAN);
 	offset += 2;
 
 	/* computer name */
-	offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name, NULL);
+	offset = display_unicode_string(tvb, pinfo, tree, offset, hf_unicode_computer_name, NULL);
 
 	/* user name */
-	offset = display_unicode_string(tvb, tree, offset, hf_user_name, NULL);
+	offset = display_unicode_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_mailslot_name, NULL);
 
 	/* account control */
 	offset = dissect_account_control(tvb, tree, offset);
@@ -581,7 +553,7 @@ dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 	}
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LMNT token */
@@ -596,12 +568,12 @@ dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 
 
 static int
-dissect_smb_no_user(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_no_user(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/* 0x0B (Announce no user on machine) */
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_computer_name, NULL);
 
 	return offset;
 }
@@ -614,19 +586,19 @@ dissect_smb_relogon_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree
 	/*** 0x0d LanMan Response to relogon request ***/
 
 	/* Major version */
-	proto_tree_add_item(tree, hf_major_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_major_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* Minor version */
-	proto_tree_add_item(tree, hf_minor_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_minor_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* OS version */
-	proto_tree_add_item(tree, hf_os_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_os_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LM token */
@@ -638,38 +610,32 @@ dissect_smb_relogon_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree
 
 
 static int
-dissect_smb_acc_update(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
+dissect_smb_acc_update(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
 	/*** 0x11  LM2.1 Announce Acc updates  ***/
 
-	guint32 Temp1, Temp2;
-
-	Temp1 = tvb_get_letohl(tvb, offset);
-	Temp2 = tvb_get_letohl(tvb, offset + 4);
-
 	/* signature */
-	proto_tree_add_text(tree, tvb, offset, 8, "Signature: 0x%08x%08x",
-		Temp1, Temp2);
+	proto_tree_add_item(tree, hf_signature, tvb, offset, 8, ENC_LITTLE_ENDIAN);
 	offset += 8;
 
 	/* date/time */
 	/* XXX - what format is this?  Neither SMB_Date/SMB_Time nor
 	   "time_t but in the local time zone" appear to be correct. */
-	proto_tree_add_item(tree, hf_date_time, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_date_time, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_computer_name, NULL);
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
+	offset = display_ms_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* update type */
-	proto_tree_add_item(tree, hf_update_type, tvb, offset, 2, TRUE);
+	proto_tree_add_item(tree, hf_update_type, tvb, offset, 2, ENC_LITTLE_ENDIAN);
 	offset += 2;
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LM token */
@@ -686,19 +652,19 @@ dissect_smb_inter_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	/* 0x0e LanMan Response to interrogate request */
 
 	/* Major version */
-	proto_tree_add_item(tree, hf_major_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_major_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* Minor version */
-	proto_tree_add_item(tree, hf_minor_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_minor_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* OS version */
-	proto_tree_add_item(tree, hf_os_version, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_os_version, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LMNT token */
@@ -712,23 +678,23 @@ dissect_smb_inter_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 
 static int
-dissect_smb_sam_logon_resp(tvbuff_t *tvb, packet_info *pinfo _U_,
+dissect_smb_sam_logon_resp(tvbuff_t *tvb, packet_info *pinfo,
 	proto_tree *tree, int offset)
 {
 	/* Netlogon command 0x13 - decode the SAM logon response from server */
 	/* Netlogon command 0x15 - decode the SAM logon response from server unknown user */
 
 	/* server name */
-	offset = display_unicode_string(tvb, tree, offset, hf_server_name, NULL);
+	offset = display_unicode_string(tvb, pinfo, tree, offset, hf_server_name, NULL);
 
 	/* user name */
-	offset = display_unicode_string(tvb, tree, offset, hf_user_name, NULL);
+	offset = display_unicode_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* domain name */
-	offset = display_unicode_string(tvb, tree, offset, hf_domain_name, NULL);
+	offset = display_unicode_string(tvb, pinfo, tree, offset, hf_domain_name, NULL);
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LMNT token */
@@ -741,7 +707,7 @@ dissect_smb_sam_logon_resp(tvbuff_t *tvb, packet_info *pinfo _U_,
 }
 
 static int
-dissect_smb_pdc_response_ads(tvbuff_t *tvb, packet_info *pinfo _U_,
+dissect_smb_pdc_response_ads(tvbuff_t *tvb, packet_info *pinfo,
 	proto_tree *tree, int offset)
 {
 	/* Netlogon command 0x17 - decode the response from PDC ADS */
@@ -751,59 +717,59 @@ dissect_smb_pdc_response_ads(tvbuff_t *tvb, packet_info *pinfo _U_,
 	offset = ((offset + 3)/4)*4;
 
 	/* unknown uint32 type */
-	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* Domain GUID */
-	proto_tree_add_item(tree, hf_domain_guid, tvb, offset, 16, TRUE);
+	proto_tree_add_item(tree, hf_domain_guid, tvb, offset, 16, ENC_NA);
 	offset += 16;
 
 	/* forest dns name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_forest_dns_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_forest_dns_name, NULL);
 
 	/* domain dns name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_domain_dns_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_domain_dns_name, NULL);
 
 	/* server dns name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_server_dns_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_server_dns_name, NULL);
 
 	/* domain name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_domain_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_domain_name, NULL);
 
 	/* server name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_server_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_server_name, NULL);
 
 	/* user name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_user_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_user_name, NULL);
 
 	/* server_site name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_server_site_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_server_site_name, NULL);
 
 	/* client_site name */
-	offset=dissect_ms_compressed_string(tvb, tree, offset, hf_client_site_name, NULL);
+	offset=dissect_ms_compressed_string(tvb, pinfo, tree, offset, hf_client_site_name, NULL);
 
 	/* unknown uint8 type */
-	proto_tree_add_item(tree, hf_unknown8, tvb, offset, 1, TRUE);
+	proto_tree_add_item(tree, hf_unknown8, tvb, offset, 1, ENC_LITTLE_ENDIAN);
 	offset += 1;
 
 	/* unknown uint32 type */
-	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* server ip */
-	proto_tree_add_item(tree, hf_server_ip, tvb, offset, 4, FALSE);
+	proto_tree_add_item(tree, hf_server_ip, tvb, offset, 4, ENC_BIG_ENDIAN);
 	offset += 4;
 
 	/* unknown uint32 type */
-	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* unknown uint32 type */
-	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_unknown32, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* NT version */
-	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
+	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, ENC_LITTLE_ENDIAN);
 	offset += 4;
 
 	/* LMNT token */
@@ -820,10 +786,9 @@ dissect_smb_unknown(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 {
 	/* display data as unknown */
 
-	proto_tree_add_text(tree, tvb, offset, -1, "Data (%u bytes)",
-	    tvb_reported_length_remaining(tvb, offset));
+	proto_tree_add_item(tree, hf_data, tvb, offset, -1, ENC_NA);
 
-	return offset+tvb_length_remaining(tvb, offset);
+	return offset+tvb_reported_length_remaining(tvb, offset);
 }
 
 #define LOGON_LM10_LOGON_REQUEST		0x00
@@ -855,87 +820,84 @@ dissect_smb_unknown(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 #define LOGON_LAST_CMD				0x19
 
 static const value_string commands[] = {
-	{LOGON_LM10_LOGON_REQUEST,	"LM1.0/LM2.0 LOGON Request"},
-	{LOGON_LM10_LOGON_RESPONSE,	"LM1.0 LOGON Response"},
-	{LOGON_LM10_QUERY_CI,		"LM1.0 Query - Centralized Initialization"},
-	{LOGON_LM10_QUERY_DI,		"LM1.0 Query - Distributed Initialization"},
-	{LOGON_LM10_RESPONSE_CI,	"LM1.0 Response - Centralized Query"},
-	{LOGON_LM10_RESPONSE_DI,	"LM1.0 Response - Distributed Initialization"},
-	{LOGON_LM20_LOGON_RESPONSE,	"LM2.0 Response to LOGON Request"},
-	{LOGON_PDC_QUERY,		"Query for PDC"},
-	{LOGON_PDC_STARTUP,		"Announce Startup of PDC"},
-	{LOGON_PDC_FAILED,		"Announce Failed PDC"},
-	{LOGON_UAS_SAM,			"Announce Change to UAS or SAM"},
-	{LOGON_NO_USER,			"Announce no user on machine"},
-	{LOGON_PDC_RESPONSE,		"Response from PDC"},
-	{LOGON_RELOGON_RESPONSE,	"LM1.0/LM2.0 Response to re-LOGON Request"},
-	{LOGON_INTERROGATE_RESPONSE,	"LM1.0/LM2.0 Response to Interrogate Request"},
-	{LOGON_LM20_RESPONSE_DURING_LOGON,"LM2.0 Response during LOGON pause"},
-	{LOGON_LM20_USER_UNKNOWN,	"LM2.0 Response - user unknown"},
-	{LOGON_LM20_ACCOUNT_UPDATE,	"LM2.0 Announce account updates"},
-	{LOGON_SAM_LOGON_REQUEST,	"SAM LOGON request from client"},
-	{LOGON_SAM_LOGON_RESPONSE,	"Response to SAM LOGON request"},
-	{LOGON_SAM_RESPONSE_DURING_LOGON,"SAM Response during LOGON pause"},
-	{LOGON_SAM_USER_UNKNOWN,	"SAM Response - user unknown"},
-	{LOGON_SAM_INTERROGATE_RESPONSE,"SAM Response to Interrogate Request"},
-	{LOGON_SAM_AD_USER_UNKNOWN,	"SAM Active Directory Response - user unknown"},
-	{LOGON_SAM_UNKNOWN_18,		"SAM unknown command 0x18"},
-	{LOGON_SAM_AD_LOGON_RESPONSE,	"Active Directory Response to SAM LOGON request"},
+	{LOGON_LM10_LOGON_REQUEST,	   "LM1.0/LM2.0 LOGON Request"},
+	{LOGON_LM10_LOGON_RESPONSE,	   "LM1.0 LOGON Response"},
+	{LOGON_LM10_QUERY_CI,		   "LM1.0 Query - Centralized Initialization"},
+	{LOGON_LM10_QUERY_DI,		   "LM1.0 Query - Distributed Initialization"},
+	{LOGON_LM10_RESPONSE_CI,	   "LM1.0 Response - Centralized Query"},
+	{LOGON_LM10_RESPONSE_DI,	   "LM1.0 Response - Distributed Initialization"},
+	{LOGON_LM20_LOGON_RESPONSE,	   "LM2.0 Response to LOGON Request"},
+	{LOGON_PDC_QUERY,		   "Query for PDC"},
+	{LOGON_PDC_STARTUP,		   "Announce Startup of PDC"},
+	{LOGON_PDC_FAILED,		   "Announce Failed PDC"},
+	{LOGON_UAS_SAM,			   "Announce Change to UAS or SAM"},
+	{LOGON_NO_USER,			   "Announce no user on machine"},
+	{LOGON_PDC_RESPONSE,		   "Response from PDC"},
+	{LOGON_RELOGON_RESPONSE,	   "LM1.0/LM2.0 Response to re-LOGON Request"},
+	{LOGON_INTERROGATE_RESPONSE,	   "LM1.0/LM2.0 Response to Interrogate Request"},
+	{LOGON_LM20_RESPONSE_DURING_LOGON, "LM2.0 Response during LOGON pause"},
+	{LOGON_LM20_USER_UNKNOWN,	   "LM2.0 Response - user unknown"},
+	{LOGON_LM20_ACCOUNT_UPDATE,	   "LM2.0 Announce account updates"},
+	{LOGON_SAM_LOGON_REQUEST,	   "SAM LOGON request from client"},
+	{LOGON_SAM_LOGON_RESPONSE,	   "Response to SAM LOGON request"},
+	{LOGON_SAM_RESPONSE_DURING_LOGON,  "SAM Response during LOGON pause"},
+	{LOGON_SAM_USER_UNKNOWN,	   "SAM Response - user unknown"},
+	{LOGON_SAM_INTERROGATE_RESPONSE,   "SAM Response to Interrogate Request"},
+	{LOGON_SAM_AD_USER_UNKNOWN,	   "SAM Active Directory Response - user unknown"},
+	{LOGON_SAM_UNKNOWN_18,		   "SAM unknown command 0x18"},
+	{LOGON_SAM_AD_LOGON_RESPONSE,	   "Active Directory Response to SAM LOGON request"},
 	{0,	NULL}
 };
 
 static int (*dissect_smb_logon_cmds[])(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset) = {
-	dissect_smb_logon_request,  /* 0x00 (LM1.0/LM2.0 LOGON Request) */
-	dissect_smb_logon_LM10_resp,/* 0x01 (LM1.0 LOGON Response)	*/
-	dissect_smb_logon_2,	    /* 0x02 (LM1.0 Query Centralized Init.)*/
-	dissect_smb_logon_2,	    /* 0x03 (LM1.0 Query Distributed Init.)*/
-	dissect_smb_logon_2,	    /* 0x04 (LM1.0 Centralized Query Resp.)*/
-	dissect_smb_logon_2,	    /* 0x05 (LM1.0 Distributed Query Resp.) */
-	dissect_smb_logon_LM20_resp,/* 0x06 (LM2.0 LOGON Response)	*/
-	dissect_smb_pdc_query,	    /* 0x07 (Query for PDC) 		*/
-	dissect_smb_pdc_startup,    /* 0x08 (Announce PDC startup)	*/
-	dissect_smb_pdc_failure,    /* 0x09 (Announce Failed PDC)	*/
-	dissect_announce_change,    /* 0x0A (Announce Change to UAS or SAM)*/
-	dissect_smb_no_user,	    /* 0x0B (Announce no user on machine)*/
-	dissect_smb_pdc_startup,    /* 0x0C (Response from PDC)		*/
-	dissect_smb_relogon_resp,   /* 0x0D (Relogon response) 		*/
-	dissect_smb_inter_resp,     /* 0x0E (Interrogate response) 	*/
-	dissect_smb_pdc_failure,    /* 0x0F (LM2.0 Resp. during LOGON pause*/
-	dissect_smb_pdc_failure,    /* 0x10 (LM 2.0 Unknown user response)*/
-	dissect_smb_acc_update,	    /* 0x11 (LM2.1 Announce Acc updates)*/
-	dissect_smb_sam_logon_req,  /* 0x12 (SAM LOGON request )	*/
-	dissect_smb_sam_logon_resp, /* 0x13 (SAM LOGON response)	*/
-	dissect_smb_unknown,        /* 0x14 (SAM Response during LOGON Pause) */
-	dissect_smb_sam_logon_resp, /* 0x15 (SAM Response User Unknown)	*/
-	dissect_smb_unknown,        /* 0x16 (SAM Response to Interrogate)*/
-	dissect_smb_pdc_response_ads,        /* 0x17 (SAM AD response User Unknown*/
-	dissect_smb_unknown,        /* 0x18 (Unknown command)		*/
-	dissect_smb_pdc_response_ads         /* 0x19 (SAM LOGON AD response)	*/
+	dissect_smb_logon_request,    /* 0x00 (LM1.0/LM2.0 LOGON Request) */
+	dissect_smb_logon_LM10_resp,  /* 0x01 (LM1.0 LOGON Response)	*/
+	dissect_smb_logon_2,	      /* 0x02 (LM1.0 Query Centralized Init.)*/
+	dissect_smb_logon_2,	      /* 0x03 (LM1.0 Query Distributed Init.)*/
+	dissect_smb_logon_2,	      /* 0x04 (LM1.0 Centralized Query Resp.)*/
+	dissect_smb_logon_2,	      /* 0x05 (LM1.0 Distributed Query Resp.) */
+	dissect_smb_logon_LM20_resp,  /* 0x06 (LM2.0 LOGON Response)	*/
+	dissect_smb_pdc_query,	      /* 0x07 (Query for PDC) 		*/
+	dissect_smb_pdc_startup,      /* 0x08 (Announce PDC startup)	*/
+	dissect_smb_pdc_failure,      /* 0x09 (Announce Failed PDC)	*/
+	dissect_announce_change,      /* 0x0A (Announce Change to UAS or SAM)*/
+	dissect_smb_no_user,	      /* 0x0B (Announce no user on machine)*/
+	dissect_smb_pdc_startup,      /* 0x0C (Response from PDC)		*/
+	dissect_smb_relogon_resp,     /* 0x0D (Relogon response) 		*/
+	dissect_smb_inter_resp,	      /* 0x0E (Interrogate response) 	*/
+	dissect_smb_pdc_failure,      /* 0x0F (LM2.0 Resp. during LOGON pause*/
+	dissect_smb_pdc_failure,      /* 0x10 (LM 2.0 Unknown user response)*/
+	dissect_smb_acc_update,	      /* 0x11 (LM2.1 Announce Acc updates)*/
+	dissect_smb_sam_logon_req,    /* 0x12 (SAM LOGON request )	*/
+	dissect_smb_sam_logon_resp,   /* 0x13 (SAM LOGON response)	*/
+	dissect_smb_unknown,	      /* 0x14 (SAM Response during LOGON Pause) */
+	dissect_smb_sam_logon_resp,   /* 0x15 (SAM Response User Unknown)	*/
+	dissect_smb_unknown,	      /* 0x16 (SAM Response to Interrogate)*/
+	dissect_smb_pdc_response_ads, /* 0x17 (SAM AD response User Unknown*/
+	dissect_smb_unknown,	      /* 0x18 (Unknown command)		*/
+	dissect_smb_pdc_response_ads  /* 0x19 (SAM LOGON AD response)	*/
 };
 
 
-static void
-dissect_smb_logon(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+static int
+dissect_smb_logon(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void* data _U_)
 {
 	int        offset = 0;
 	guint8     cmd;
 	proto_tree *smb_logon_tree = NULL;
 	proto_item *item = NULL;
 
-	if (check_col(pinfo->cinfo, COL_PROTOCOL))
-		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SMB_NETLOGON");
-	if (check_col(pinfo->cinfo, COL_INFO))
-		col_clear(pinfo->cinfo, COL_INFO);
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "SMB_NETLOGON");
+	col_clear(pinfo->cinfo, COL_INFO);
 
 	/* get the Command field */
-   	cmd = tvb_get_guint8(tvb, offset);
+	cmd = tvb_get_guint8(tvb, offset);
 
-	if (check_col(pinfo->cinfo, COL_INFO))
-		col_add_str(pinfo->cinfo, COL_INFO, val_to_str(cmd, commands, "Unknown Command:%02x") );
+	col_add_str(pinfo->cinfo, COL_INFO, val_to_str(cmd, commands, "Unknown Command:%02x") );
 
-    	if (tree) {
+	if (tree) {
 		item = proto_tree_add_item(tree, proto_smb_logon, tvb,
-			offset,	-1, FALSE);
+			offset,	-1, ENC_NA);
 
 		smb_logon_tree = proto_item_add_subtree(item, ett_smb_logon);
 	}
@@ -948,13 +910,14 @@ dissect_smb_logon(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	offset += 1;
 
 	if (cmd<LOGON_LAST_CMD) {
-		offset = (dissect_smb_logon_cmds[cmd])(tvb, pinfo,
+		(dissect_smb_logon_cmds[cmd])(tvb, pinfo,
 		    smb_logon_tree, offset);
 	} else {
 		/* unknown command */
-		offset = dissect_smb_unknown(tvb, pinfo, smb_logon_tree,
+		dissect_smb_unknown(tvb, pinfo, smb_logon_tree,
 		    offset);
 	}
+	return tvb_captured_length(tvb);
 }
 
 void
@@ -1039,6 +1002,10 @@ proto_register_smb_logon( void)
 			{ "Workstation OS Version", "smb_netlogon.os_version", FT_UINT8, BASE_DEC,
 			  NULL, 0, "SMB NETLOGON Workstation OS Version", HFILL }},
 
+		{ &hf_signature,
+			{ "Signature", "smb_netlogon.signature", FT_UINT64, BASE_HEX,
+			  NULL, 0, NULL, HFILL }},
+
 		{ &hf_date_time,
 			{ "Date/Time", "smb_netlogon.date_time", FT_UINT32, BASE_DEC,
 			  NULL, 0, "SMB NETLOGON Date/Time", HFILL }},
@@ -1051,6 +1018,10 @@ proto_register_smb_logon( void)
 			{ "Request Count", "smb_netlogon.request_count", FT_UINT16, BASE_DEC,
 			  NULL, 0, "SMB NETLOGON Request Count", HFILL }},
 
+		{ &hf_account_control,
+			{ "Account control", "smb_netlogon.flags", FT_UINT32, BASE_HEX,
+			  NULL, 0, NULL, HFILL }},
+
 		{ &hf_flags_autolock,
 			{ "Autolock", "smb_netlogon.flags.autolock", FT_BOOLEAN, 32,
 			TFS(&tfs_flags_autolock), ACC_FLAG_AUTO_LOCKED, "SMB NETLOGON Account Autolock", HFILL}},
@@ -1124,20 +1095,20 @@ proto_register_smb_logon( void)
 			  NULL, 0, "SMB NETLOGON Large Serial Number", HFILL }},
 
 		{ &hf_nt_date_time,
-			{ "NT Date/Time", "smb_netlogon.nt_date_time", FT_ABSOLUTE_TIME, BASE_NONE,
+			{ "NT Date/Time", "smb_netlogon.nt_date_time", FT_ABSOLUTE_TIME, ABSOLUTE_TIME_LOCAL,
 			  NULL, 0, "SMB NETLOGON NT Date/Time", HFILL }},
 
 		{ &hf_unknown8,
 			{ "Unknown", "smb_netlogon.unknown", FT_UINT8, BASE_HEX,
-			  NULL, 0, "Unknown", HFILL }},
+			  NULL, 0, NULL, HFILL }},
 
 		{ &hf_unknown32,
 			{ "Unknown", "smb_netlogon.unknown", FT_UINT32, BASE_HEX,
-			  NULL, 0, "Unknown", HFILL }},
+			  NULL, 0, NULL, HFILL }},
 
 		{ &hf_domain_guid,
-			{ "Domain GUID", "smb_netlogon.domain.guid", FT_BYTES, BASE_HEX,
-			   NULL, 0x0, "Domain GUID", HFILL }},
+			{ "Domain GUID", "smb_netlogon.domain.guid", FT_BYTES, BASE_NONE,
+			   NULL, 0x0, NULL, HFILL }},
 
 		{ &hf_server_ip, {
 			"Server IP", "smb_netlogon.server_ip", FT_IPv4, BASE_NONE,
@@ -1150,6 +1121,10 @@ proto_register_smb_logon( void)
 		{ &hf_client_site_name,
 			{ "Client Site Name", "smb_netlogon.client_site_name", FT_STRING, BASE_NONE,
 			  NULL, 0, "SMB NETLOGON Client Site Name", HFILL }},
+
+		{ &hf_data,
+			{ "Data", "smb_netlogon.data", FT_BYTES, BASE_NONE,
+			  NULL, 0, NULL, HFILL }},
 	};
 
 	static gint *ett[] = {
@@ -1158,11 +1133,23 @@ proto_register_smb_logon( void)
 		&ett_smb_db_info
 	};
 
-   	proto_smb_logon = proto_register_protocol(
-   		"Microsoft Windows Logon Protocol (Old)", "SMB_NETLOGON", "smb_netlogon");
+	proto_smb_logon = proto_register_protocol("Microsoft Windows Logon Protocol (Old)", "SMB_NETLOGON", "smb_netlogon");
 
 	proto_register_field_array(proto_smb_logon, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
 	register_dissector("smb_netlogon", dissect_smb_logon, proto_smb_logon);
 }
+
+/*
+ * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
+ *
+ * Local variables:
+ * c-basic-offset: 8
+ * tab-width: 8
+ * indent-tabs-mode: t
+ * End:
+ *
+ * vi: set shiftwidth=8 tabstop=8 noexpandtab:
+ * :indentSize=8:tabSize=8:noTabs=false:
+ */
