@@ -1,31 +1,24 @@
 /* strutil.h
  * String utility definitions
  *
- * $Id: strutil.h 29478 2009-08-19 18:37:13Z etxrab $
- *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __STRUTIL_H__
 #define __STRUTIL_H__
 
-/* ... thus, config.h needs to be #included */
+#include "ws_symbol_export.h"
+
+#include <epan/wmem_scopes.h>
+#include <wsutil/str_util.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
 /** @file
  * String handling and conversion utilities.
@@ -50,44 +43,10 @@ const guchar *find_line_end(const guchar *data, const guchar *dataend,
  * @param next_token Receives the location of the next token
  * @return 0 if there is no next token.
  */
+WS_DLL_PUBLIC
 int        get_token_len(const guchar *linep, const guchar *lineend,
     const guchar **next_token);
 
-/** Given a string, generate a string from it that shows non-printable
- *  characters as C-style escapes, and return a pointer to it.
- *
- * @param line A pointer to the input string
- * @param len The length of the input string
- * @return A pointer to the formatted string
- *
- * @see tvb_format_text()
- */
-gchar*     format_text(const guchar *line, size_t len);
-
-gchar*     format_text_wsp(const guchar *line, size_t len);
-
-/** Turn an array of bytes into a string showing the bytes in hex.
- *
- * @param bd A pointer to the byte array
- * @param bd_len The length of the byte array
- * @return A pointer to the formatted string
- *
- * @see bytes_to_str_punct()
- */
-gchar*     bytes_to_str(const guint8 *bd, int bd_len);
-
-/** Turn an array of bytes into a string showing the bytes in hex,
- *  separated by a punctuation character.
- *
- * @param bd A pointer to the byte array
- * @param bd_len The length of the byte array
- * @param punct The punctuation character
- * @return A pointer to the formatted string
- *
- * @see bytes_to_str()
- */
-gchar*     bytes_to_str_punct(const guint8 *bd, int bd_len, gchar punct);
-
 /** Turn a string of hex digits with optional separators (defined by
  *  is_byte_sep() into a byte array.
  *
@@ -98,33 +57,69 @@ gchar*     bytes_to_str_punct(const guint8 *bd, int bd_len, gchar punct);
  *        bytes.
  * @return True if the string was converted successfully
  */
+WS_DLL_PUBLIC
 gboolean   hex_str_to_bytes(const char *hex_str, GByteArray *bytes,
     gboolean force_separators);
 
+/* Turn a string of hex digits with optional separators (defined by encoding)
+ * into a byte array. Unlike hex_str_to_bytes(), this will read as many hex-char
+ * pairs as possible and not error if it hits a non-hex-char; instead it just ends
+ * there. (i.e., like strtol()/atoi()/etc.) But it must see two hex chars at the
+ * beginning or it will return FALSE.
+ *
+ * @param hex_str The string of hex digits.
+ * @param bytes The GByteArray that will receive the bytes.  This
+ *        must be initialized by the caller.
+ * @param endptr if not NULL, is set to the char after the last hex character consumed.
+ * @param encoding set to one or more bitwise-or'ed ENC_SEP_* (see proto.h)
+ * @param fail_if_partial If set to TRUE, then the conversion fails if the whole
+ *    hex_str is not consumed.
+ * @return FALSE only if no bytes were generated; or if fail_if_partial is TRUE
+ *    and the entire hex_str was not consumed.
+ *
+ * If no ENC_SEP_* is set, then no separators are allowed. If multiple ENC_SEP_* are
+ * bit-or'ed, any of them can be a separator, but once the separator is seen then
+ * only its same type is accepted for the rest of the string. (i.e., it won't convert
+ * a "01:23-4567" even if ENC_SEP_COLON|ENC_SEP_DASH|ENC_SEP_NONE is passed in)
+ *
+ * This is done this way because it's likely a malformed scenario if they're mixed,
+ * and this routine is used by dissectors via tvb_get_string_XXX routines.
+ */
+WS_DLL_PUBLIC
+gboolean hex_str_to_bytes_encoding(const char *hex_str, GByteArray *bytes, const char **endptr,
+                                   const guint encoding, const gboolean fail_if_partial);
+
+/** Turn an RFC 3986 percent-encoded array of characters, not necessarily
+ * null-terminated, into a byte array.
+ *
+ * @param uri_str The string of hex digits.
+ * @param bytes The GByteArray that will receive the bytes.  This
+ *        must be initialized by the caller.
+ * @return True if the string was converted successfully
+ */
+WS_DLL_PUBLIC
+gboolean   uri_to_bytes(const char *uri_str, GByteArray *bytes, size_t len);
+
 /** Turn an RFC 3986 percent-encoded string into a byte array.
  *
  * @param uri_str The string of hex digits.
  * @param bytes The GByteArray that will receive the bytes.  This
  *        must be initialized by the caller.
  * @return True if the string was converted successfully
- * @see format_uri()
  */
+WS_DLL_PUBLIC
 gboolean   uri_str_to_bytes(const char *uri_str, GByteArray *bytes);
 
-/** Turn a byte array into an RFC 3986 percent-encoded string.
+/** Turn a OID string representation (dot notation) into a byte array.
  *
+ * @param oid_str The OID string (dot notaion).
  * @param bytes The GByteArray that will receive the bytes.  This
  *        must be initialized by the caller.
- * @param reserved_chars Normally the "gen-delims" and "sub-delims"
- *        from RFC 3986 (":/?#[]@" and "!$&'()*+,;=" respectively)
- *        plus space (hex value 20) are treated as reserved characters.
- *        If this variable is non-NULL, its contents will be used
- *        instead.
- * @note Any non-printing character determined by isprint(), along
- *       with the % character itself are always reserved.
- * @see uri_str_to_bytes(),  format_text(), isprint()
+ * @param is_absolute True if this is an absolute OID; false for relative OID.
+ * @return True if the string was converted successfully
  */
-gchar*     format_uri(const GByteArray *bytes, const gchar *reserved_chars);
+WS_DLL_PUBLIC
+gboolean   rel_oid_str_to_bytes(const char *oid_str, GByteArray *bytes, gboolean is_absolute);
 
 /** Turn a OID string representation (dot notation) into a byte array.
  *
@@ -133,6 +128,7 @@ gchar*     format_uri(const GByteArray *bytes, const gchar *reserved_chars);
  *        must be initialized by the caller.
  * @return True if the string was converted successfully
  */
+WS_DLL_PUBLIC
 gboolean   oid_str_to_bytes(const char *oid_str, GByteArray *bytes);
 
 /**
@@ -141,9 +137,10 @@ gboolean   oid_str_to_bytes(const char *oid_str, GByteArray *bytes);
  * @param ba The byte array to be copied.
  * @return If ba exists, a freshly allocated copy.  NULL otherwise.
  *
- * XXX - Should this be in strutil.c?
+ * @todo - Should this be in strutil.c?
  */
-GByteArray *byte_array_dup(GByteArray *ba);
+WS_DLL_PUBLIC
+GByteArray *byte_array_dup(const GByteArray *ba);
 
 /**
  * Compare the contents of two GByteArrays
@@ -154,8 +151,9 @@ GByteArray *byte_array_dup(GByteArray *ba);
  *         their contents are equal, returns TRUE.  Otherwise, returns
  *         FALSE.
  *
- * XXX - Should this be in strutil.c?
+ * @todo - Should this be in strutil.c?
  */
+WS_DLL_PUBLIC
 gboolean byte_array_equal(GByteArray *ba1, GByteArray *ba2);
 
 
@@ -165,27 +163,9 @@ gboolean byte_array_equal(GByteArray *ba1, GByteArray *ba2);
  * @param unescaped The unescaped string
  * @return An XML-escaped representation of the input string
  */
+WS_DLL_PUBLIC
 gchar*     xml_escape(const gchar *unescaped);
 
-/**
- * Return the first occurrence of needle in haystack.
- * Algorithm copied from GNU's glibc 2.3.2 memcmp()
- *
- * @param haystack The data to search
- * @param haystack_len The length of the search data
- * @param needle The string to look for
- * @param needle_len The length of the search string
- * @return A pointer to the first occurrence of "needle" in
- *         "haystack".  If "needle" isn't found or is NULL, or if
- *         "needle_len" is 0, NULL is returned.
- */
-const guint8 * epan_memmem(const guint8 *haystack, guint haystack_len,
-		const guint8 *needle, guint needle_len);
-
-/* Surround a string or a macro, resolved to a string, with double quotes */
-#define _STRINGIFY(a)           # a
-#define STRINGIFY(a)            _STRINGIFY(a)
-
 /** Scan a string to make sure it's valid hex.
  *
  * @param string The string to validate
@@ -193,6 +173,7 @@ const guint8 * epan_memmem(const guint8 *haystack, guint haystack_len,
  * @return A pointer to a buffer containing the converted raw bytes.  This
  *         buffer must be g_free()d by the caller.
  */
+WS_DLL_PUBLIC
 guint8 * convert_string_to_hex(const char *string, size_t *nbytes);
 
 /** Prep a string for case-sensitive vs case-insensitive searching.
@@ -203,26 +184,29 @@ guint8 * convert_string_to_hex(const char *string, size_t *nbytes);
  * an uppercased version if not.  In either case the string must be g_free()d
  * by the caller.
  */
+WS_DLL_PUBLIC
 char * convert_string_case(const char *string, gboolean case_insensitive);
 
-/** Finds the first occurence of string 'needle' in string 'haystack'.
- *  The matching is done in a case insensitive manner.
- *
- * @param haystack The string possibly containing the substring
- * @param needle The substring to be searched
- * @return A pointer into 'haystack' where 'needle' is first found.
- *   Otherwise it returns NULL.
- */
-char * epan_strcasestr(const char *haystack, const char *needle);
+WS_DLL_PUBLIC
+void IA5_7BIT_decode(unsigned char * dest, const unsigned char* src, int len);
 
-/** Guarantee a non-null string.
- *
- * @param string The string to check
- * @return A pointer 'string' if it's non-null, otherwise "[NULL]".
- */
-const char * string_or_null(const char *string);
+#define FORMAT_LABEL_REPLACE_SPACE      (0x1 << 0)
 
-int escape_string_len(const char *string);
-char * escape_string(char *dst, const char *string);
+WS_DLL_PUBLIC
+size_t ws_label_strcpy(char *label_str, size_t bufsize, gsize pos, const uint8_t *str, int flags);
+
+WS_DLL_PUBLIC
+size_t ws_label_strcat(char *label_str, size_t bufsize, const uint8_t *str, int flags);
+
+/*
+ * Check name is valid. This covers names for display filter fields, dissector
+ * tables, preference modules, etc. Lower case is preferred.
+ */
+WS_DLL_LOCAL guchar
+module_check_valid_name(const char *name, gboolean lower_only);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif /* __STRUTIL_H__ */
