@@ -1,6 +1,4 @@
-/* nettl.h
- *
- * $Id: nettl.h 23290 2007-10-27 15:44:12Z etxrab $
+/** @file
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -8,24 +6,15 @@
  * Enhancements by Mark C. Brown <mbrown@hp.com>
  * Copyright (C) 2003, 2005 Hewlett-Packard Development Company, L.P.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __NETTL_H__
 #define __NETTL_H__
 
+#include <glib.h>
+#include <wiretap/wtap.h>
+
 /* nettl subsystems are defined in /etc/nettlgen.conf */
 
 #define NETTL_SUBSYS_NS_LS_LOGGING	0
@@ -92,6 +81,10 @@
 #define NETTL_SUBSYS_IGELAN		252
 #define NETTL_SUBSYS_IETHER		253
 #define NETTL_SUBSYS_IXGBE		265
+#define NETTL_SUBSYS_ICXGBE		271
+#define NETTL_SUBSYS_IEXGBE		275
+#define NETTL_SUBSYS_IOCXGBE		277
+#define NETTL_SUBSYS_IQXGBE		278
 
 /* FDDI cards */
 #define NETTL_SUBSYS_HPPB_FDDI		95
@@ -103,6 +96,10 @@
 #define NETTL_SUBSYS_TOKEN		31
 #define NETTL_SUBSYS_PCI_TR		187
 
+/* Accelerated Virtual I/O (AVIO) drivers */
+#define NETTL_SUBSYS_HSSN		269
+#define NETTL_SUBSYS_IGSSN		270
+
 /* from /usr/include/sys/subsys_id.h */
 
 #define NETTL_HDR_HDRIN                 0x80000000
@@ -117,8 +114,8 @@
 #define NETTL_HDR_PTOP                  0x00400000
 #define NETTL_HDR_SUBSYSTEM_BITS_MASK   0x000fffff
 
-int nettl_open(wtap *wth, int *err, gchar **err_info);
-gboolean nettl_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err);
-int nettl_dump_can_write_encap(int encap);
+#define NETTL_HDR_PDU_MASK              0x30000000
+
+wtap_open_return_val nettl_open(wtap *wth, int *err, gchar **err_info);
 
 #endif
